import pytest
from quart import Quart
from werkzeug.datastructures import HeaderSet

from quart_cors import route_cors


@pytest.fixture(name="app", scope="function")
def _app() -> Quart:
    app = Quart(__name__)

    @app.route("/")
    @route_cors()
    async def index() -> str:
        return "Hello"

    return app


# These tests are based on https://www.w3.org/TR/cors section 6.2, and
# follow the logic given.


@pytest.mark.asyncio
async def test_no_origin(app: Quart) -> None:
    test_client = app.test_client()
    response = await test_client.options("/")
    assert "Access-Control-Allow-Origin" not in response.headers


@pytest.mark.asyncio
@pytest.mark.parametrize("origin", ["http://notquart.com", "http://Quart.com"])
async def test_origin_doesnt_match(app: Quart, origin: str) -> None:
    test_client = app.test_client()
    app.config["QUART_CORS_ALLOW_ORIGIN"] = ["http://quart.com"]
    response = await test_client.options("/", headers={"Origin": origin})
    assert "Access-Control-Allow-Origin" not in response.headers


@pytest.mark.asyncio
async def test_request_method_doesnt_match(app: Quart) -> None:
    test_client = app.test_client()
    app.config["QUART_CORS_ALLOW_METHODS"] = ["GET", "POST"]
    response = await test_client.options(
        "/", headers={"Origin": "http://quart.com", "Access-Control-Request-Method": "DELETE"}
    )
    assert response.access_control_allow_origin == "*"
    assert "Access-Control-Allow-Headers" not in response.headers


@pytest.mark.asyncio
async def test_request_method_match(app: Quart) -> None:
    test_client = app.test_client()
    app.config["QUART_CORS_ALLOW_METHODS"] = ["GET", "POST"]
    response = await test_client.options(
        "/", headers={"Origin": "http://quart.com", "Access-Control-Request-Method": "POST"}
    )
    assert response.access_control_allow_origin == "*"
    assert response.access_control_allow_methods == HeaderSet(["GET", "POST"])


@pytest.mark.asyncio
async def test_request_headers(app: Quart) -> None:
    test_client = app.test_client()
    app.config["QUART_CORS_ALLOW_HEADERS"] = ["X-Match", "X-Other"]
    response = await test_client.options(
        "/",
        headers={
            "Origin": "http://quart.com",
            "Access-Control-Request-Method": "POST",
            "Access-Control-Request-Headers": "X-Match, X-No-Match",
        },
    )
    assert response.access_control_allow_origin == "*"
    assert response.access_control_allow_headers == HeaderSet(["X-Match"])



import re
from datetime import timedelta
from typing import Pattern, Union

import pytest
from quart import Blueprint, Quart
from werkzeug.datastructures import HeaderSet

from quart_cors import cors, route_cors


@pytest.fixture(name="route_cors_app")
def _route_cors_app() -> Quart:
    app = Quart(__name__)

    @app.route("/")
    @route_cors(max_age=timedelta(seconds=5))
    async def index() -> str:
        return "Hello"

    return app


@pytest.mark.asyncio
async def test_simple_cross_origin_request(route_cors_app: Quart) -> None:
    test_client = route_cors_app.test_client()
    response = await test_client.get("/", headers={"Origin": "https://quart.com"})
    assert response.access_control_allow_origin == "*"


@pytest.mark.asyncio
async def test_preflight_request(route_cors_app: Quart) -> None:
    test_client = route_cors_app.test_client()
    response = await test_client.options(
        "/", headers={"Origin": "https://quart.com", "Access-Control-Request-Method": "DELETE"}
    )
    assert response.access_control_allow_origin == "*"
    assert response.access_control_allow_methods == HeaderSet(
        ["GET", "HEAD", "POST", "OPTIONS", "PUT", "PATCH", "DELETE"]
    )
    assert response.access_control_max_age == 5


@pytest.mark.asyncio
@pytest.mark.parametrize("kind", ("no-header", "empty-header"))
async def test_bad_preflight_request(route_cors_app: Quart, kind: str) -> None:
    test_client = route_cors_app.test_client()
    # Missing Access-Control-Request-Method header
    headers = {"Origin": "https://quart.com"}
    if kind == "empty-header":
        headers["Access-Control-Request-Method"] = ""
    response = await test_client.options("/", headers=headers)
    assert response.access_control_allow_origin == "*"
    assert response.access_control_allow_methods is None
    assert response.access_control_max_age is None


@pytest.mark.asyncio
async def test_app_cors() -> None:
    app = Quart(__name__)

    @app.route("/")
    async def index() -> str:
        return "Hello"

    app = cors(app)

    test_client = app.test_client()
    response = await test_client.get("/", headers={"Origin": "https://quart.com"})
    assert response.access_control_allow_origin == "*"


@pytest.mark.asyncio
async def test_blueprint_cors() -> None:
    app = Quart(__name__)

    blueprint = Blueprint("name", __name__)

    blueprint = cors(blueprint)

    @blueprint.route("/")
    async def index() -> str:
        return "Hello"

    app.register_blueprint(blueprint)

    test_client = app.test_client()
    response = await test_client.get("/", headers={"Origin": "https://quart.com"})
    assert response.access_control_allow_origin == "*"


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "allowed_origin, expected",
    [
        ("*", "*"),
        ("https://quart.com", "https://quart.com"),
        (re.compile(r"https:\/\/.*\.?quart\.com"), "https://quart.com"),
    ],
)
async def test_regex_matching(allowed_origin: Union[Pattern, str], expected: str) -> None:
    app = Quart(__name__)
    app.config["QUART_CORS_ALLOW_ORIGIN"] = [allowed_origin]

    @app.route("/")
    async def index() -> str:
        return "Hello"

    app = cors(app)

    test_client = app.test_client()
    response = await test_client.get("/", headers={"Origin": "https://quart.com"})
    assert response.access_control_allow_origin == expected



import pytest
from quart import Quart, websocket
from quart.testing import WebsocketResponse

from quart_cors import websocket_cors


@pytest.fixture(name="websocket_cors_app")
def _websocket_cors_app() -> Quart:
    app = Quart(__name__)

    @app.websocket("/")
    @websocket_cors(allow_origin=["https://quart.com"])
    async def ws() -> None:
        await websocket.send(b"a")

    return app


@pytest.mark.asyncio
async def test_websocket_allowed(websocket_cors_app: Quart) -> None:
    test_client = websocket_cors_app.test_client()
    async with test_client.websocket(
        "/", headers={"Origin": "https://quart.com"}
    ) as test_websocket:
        data = await test_websocket.receive()
    assert data == b"a"  # type: ignore


@pytest.mark.asyncio
async def test_websocket_blocked(websocket_cors_app: Quart) -> None:
    test_client = websocket_cors_app.test_client()
    try:
        async with test_client.websocket("/") as test_websocket:
            await test_websocket.send(b"a")
    except WebsocketResponse as error:
        assert error.response.status_code == 400
